#include "COM.h"
#include <Arduino.h>

// le constructeur
COM::COM(int cs, int baudRate, int idMode): MCP_CAN(cs){
CS = cs;
switch (baudRate) {
case 50 : {debit = CAN_50KBPS; break;}
case 100 :{debit = CAN_100KBPS; break;}
case 250 : {debit = CAN_250KBPS; break;}
case 500 : {debit = CAN_500KBPS; break;}
case 1000 : {debit = CAN_1000KBPS; break;}
}
ID_mode = idMode;
}

// le destructeur
COM::~COM(){}

/************************************************
INIT : cette initialisation simplifiee permet de
recevoir des messages uniquement sur l'ID du passage a niveau
le masque est fixe a 0xFFF et le filtre avec l'ID du passage
Cela limite le nombre de parametres a rentrer
parmetres :
    filtre : l'ID du passage a niveau
    ID : l'ID du distant (TCO), le passage n'a apriori pas a communiquer avec d'autre
**************************************************************/

bool COM::init(int filtre, int ID) {
    IDlocal = filtre;
    IDdistant = ID;
    return init(MASQUE,filtre,filtre,MASQUE,filtre,filtre,filtre,filtre);
    }

/**********************************************************************
INIT : l'initialisation complete du bus CAN
*************************************************************************/

bool COM::init(int masque0, int filtre0, int filtre1, int masque1, int filtre2, int filtre3, int filtre4, int filtre5) {
int repetition = TENTATIVE_CONNECTION;
while (repetition > 0) {
        if (CAN_OK == begin(debit)){break;}else {repetition--;}
        delay(200);
    }
if (repetition == 0) return false;
init_Mask(0,ID_mode, masque0);
init_Filt(0,ID_mode, filtre0);
init_Filt(1,ID_mode, filtre1);
init_Mask(1,ID_mode, masque1);
init_Filt(2,ID_mode, filtre2);
init_Filt(3,ID_mode, filtre3);
init_Filt(4,ID_mode, filtre4);
init_Filt(5,ID_mode, filtre5);
return true;
}

/*******************************************************
LIRECODE : permet de recevoir un code depuis le TCO
parametre :
    code : le code lu du message
********************************************************/

bool COM::lireCode (unsigned char &code){
    if (indexLecture != indexEcriture) {              // s'il y a un message en attente
      code = BufferReception[indexLecture].buf[0];       // on le lit et le range dans la variable globale "message"
      indexLecture = (indexLecture+1) % NB_MESSAGES;   // on avance circulairement l'index de lecture
      return true;                              // renvoie true si message a traiter
    }
  return false;                                 // renvoie false si pas de message a traiter
}

/********************************************************
ECRIRECODE : permet d'envoyer au TCO un code
parametre : code : le code a envoyer
*******************************************************/

int COM::ecrireCode(unsigned char code) {
unsigned char buf[MAX_CHAR_IN_MESSAGE];
buf[0] = code;
return sendMsgBuf(IDdistant, ID_mode,1, buf);
}

/*********************************************************
RECEPTIONMESSAGE : methode interne qui stocke les messages du bus dans un buffer circulaire
***********************************************************/

void COM::receptionMessage(){
  long unsigned int Id;           // identificateur
  unsigned char Len = 0;      // longueur
  unsigned char Buf[8];       // donnees
  while (CAN_MSGAVAIL == checkReceive()) {   // tant qu'il y des messages a recevoir
    Serial.println(F("on recoit des messages"));
    readMsgBufID(&Id,&Len,Buf);                    // lecture de l'ID et la longueur et des donnees
    BufferReception[indexEcriture].id = Id;           // on sauve l'identificateur
    BufferReception[indexEcriture].len = Len;         // on sauve la longueur
    for (byte index = 0; index < Len; index++){   // et les donnees dans le buffer de recption
            BufferReception[indexEcriture].buf[index] = Buf[index];
    }
    indexEcriture = (indexEcriture+1) % NB_MESSAGES;     // on avance circulairement d'un cran l'index de lecture
  }
}


