#include "passage.h"

#define UNIPOLE 1
#define DIPOLE 2
#define QUADRIPOLE 4

// les constructeurs

passage::passage() {}

/*******************************************************************
parametres :
    feux : une instance de feux
************************************************************************/

passage::passage(feux& fanal) {
_feux = &fanal;
flash = true;
}

/*******************************************************************
parametres :
    feux : une instance de feux
    name : nom du passage a niveau
************************************************************************/

passage::passage(feux& fanal, char* nom): _nom(nom) {
_feux = &fanal;
flash = true;
}


/*******************************************************************
parametres :
    barriere : une instance de barriere de type relais ou bobine
************************************************************************/

passage::passage(barriere& barrier) {
_barriere = &barrier;
barre = true;
}

/*******************************************************************
parametres :
    barriere : une instance de barriere de type relais ou bobine
    name : nom du passage a niveau
************************************************************************/

passage::passage(barriere& barrier, char* nom): _nom(nom) {
_barriere = &barrier;
barre = true;
}

/*******************************************************************
parametres :
    barriere : une instance de barriere de type relais ou bobine
    feux : une instance de feux
************************************************************************/

passage::passage(barriere& barrier, feux& fanal) {
_barriere = &barrier;
barre = true;
_feux = &fanal;
flash = true;
}

/************************************************************************
parametres :
    barriere : une instance de barriere de type relais ou bobine
    feux : une instance de feux
    name : le nom du passage a niveau
*******************************************************************/

passage::passage(barriere& barrier,feux& fanal, char* nom): _nom(nom){
_barriere = &barrier;
barre = true;
_feux = &fanal;
flash = true;
}

// le destructeur
passage::~passage(){}

/********************************************************
ADD : ajoute une voie au passage a niveau
parametre :
    info : une instance de voie
retour : bool (vrai si reussi, faux si memoire saturee)
**********************************************************/

bool passage::add(voie &rail){
    element *nouveau = new element;                         // on cree un nouvel element
    if (nouveau == nullptr) {delete(nouveau);return false;} // si NULL, memoire saturee
    nouveau->_voie = &rail;                           // mise a jour de la donnee
    if (debut == nullptr) {debut = nouveau;}                // c'est la premiere donnee
    else {fin->suivant = nouveau;}                          // sinon mis en fin de file
    fin = nouveau;                                          // on avance fin d'un cran
    return true;
}

/*********************************************************
ADDRESET : ajoute la pin de reset au passage a niveau
********************************************************/

void passage::addReset(int pin) {
pinReset = pin;
pinMode(pinReset,OUTPUT);
digitalWrite(pinReset,HIGH);
 }

/******************************************************************
ADDCAN : m�thode pour ajouter le bus CAN au passage � niveau
parametres :
    cs : la broche CS = 10 UNO/NANO/MINI, CS = 53 MEGA
    debit : debit du bus (50, 100, 250, 500, 1000)
    mode : 0 : normal, 1 : etendu
    IDlocal : ID can du passage a niveau
    IDdistant : ID can du TCO
*************************************************************************/

void passage::addCAN(int cs, int debit,int mode,int IDlocal,int IDdistant){
can = new COM(cs,debit,mode);
can->init(IDlocal,IDdistant);
}

/*********************************************************************
UPDATECAN : v�rifie les receptions du bus
*********************************************************************/

void passage::updateCAN() {can->receptionMessage();}

/*************************************************************
TESTCAN : si message en attente on le traite
***************************************************************/

void passage::testCAN(){
  recept = false;
    if (can->lireCode(data)){
        switch (data) {
            case RESET : {
                        can->ecrireCode(RESET_CONF);
                        digitalWrite(pinReset, LOW);
                        delay(20);
                        digitalWrite(pinReset,HIGH);
                        break;}
        case TRIAGE_ON : {
                        actionR = "ACTIF";
                        recept = true;
                        etatTriage = true;
                        if (!triageCan) {can->ecrireCode(TRIAGE_ON_CONF);triageCan = true;}
                        break;}
        case TRIAGE_OFF : {
                        actionR = "INACTIF";
                        recept = true;
                        etatTriage = false;
                        if (triageCan) {can->ecrireCode(TRIAGE_OFF_CONF); triageCan = false;}
                        break;}
            }
    }
}

void passage::reception() {if (recept) {Serial.print(F("PAN <== TCO : MODE TRIAGE "));Serial.println(actionR);}}

/**********************************************************************
PREMIER : initialise le parcours de la liste en renvoyant la voie en
tete de liste et en mettant a jour l'indexation
paramtres :
    **info : renvoie l'adresse de la voie courante
retour : bool (vrai si liste non vide, faux sinon)
******************************************************************/

bool passage::premier(voie** rail) {
    if (debut == nullptr){init = false; return false;}    // si vide rien a faire
    *rail = debut->_voie;                // on renvoie la voie courante
    index = debut->suivant;             // on avance l'index
    init = true;                        // initialisation du parcours effectuee
    return true;
}

/***************************************************************
SUIVANT :   renvoie la voie courante dans le parcours de la liste
et met a jour l'indexation
paramtres :
    **info : renvoie l'adresse de la voie courante
retour : bool (vrai si pas fin de liste, faux sinon)
*********************************************************************/

bool passage::suivant(voie** rail) {
    if (index == nullptr) {init = false; return false;} // si fin de file ou non initialisation
    *rail = index->_voie;             // on renvoie la voie courante
    index = index->suivant;         // on avance l'index
    return true;
}

/***************************************************************************
affiche la configuration du passage a niveau (si mode console)
*************************************************************************/

void passage::print() {
  voie* rail;
  // pour le passage a niveau lui meme
  Serial.print(F("passage : ")); Serial.println(_nom);
  Serial.print(F("etat du passage : ")); if (_etat == 0) {Serial.println(F("libre"));}else {Serial.println(F("occupe"));}
  // pour les feux associes (si dans configuration)
  if (flash) {Serial.print(F("feux routiers sur pin")); Serial.print(_feux->broche());Serial.print(F(", etat des feux : "));if (_feux->etatON()){Serial.println(F("allumes"));}else {Serial.println(F("eteints"));}}
  // pour la barriere du passage a niveau (si dans configuration)
  if (barre) {
        switch (_barriere->nombrePoles()) {
        case UNIPOLE : {Serial.print(F("barrieres : pin pin")); Serial.print(_barriere->broche(1));break;}
        case DIPOLE : { Serial.print(F("barrieres : pin ouvrante pin")); Serial.print(_barriere->broche(1));
                        Serial.print(F(", pin fermante pin"));Serial.print(_barriere->broche(2)); break;}
        case QUADRIPOLE : { Serial.print(F("barrieres : pin1 pin")); Serial.print(_barriere->broche(1));
                            Serial.print(F(", pin2 pin")); Serial.print(_barriere->broche(2));
                            Serial.print(F(", pin3 pin")); Serial.print(_barriere->broche(3));
                            Serial.print(F(", pin4 pin")); Serial.print(_barriere->broche(4)); break;}
        }
       Serial.print(F(", position des barrieres : "));if (_barriere->fermee()) {Serial.println(F("fermees"));}else {Serial.println(F("ouvertes"));}
       }
  Serial.println();
  // on parcours la liste des voies
     if (premier(&rail)){                     // on affiche les configurations des voies
                rail->print();
            while (suivant(&rail)){      // idem pour les suivantes
                Serial.println();
                    rail->print();
                    }
              }
}

/**********************************************************************
boucle sur les changements survenus sur les voies
****************************************************************************/

void passage::Loop() {
    testCAN();                                                  // on traite les messages CAN en attente
    if (!etatTriage){                                           // si pas triage on joue l'algo
        voie* rail;
        _etat = 0;                                             // par defaut le passage est libre
        ddeFermeture = false;
        if (premier(&rail)){                                    // on balaie toutes les voies
                   rail->Loop();                                      // on verifie les demandes de fermeture
                    if (rail->demandeFermeture()){ddeFermeture = true;}
                    _etat = _etat + rail->etat();                    // on prend l'etat de la voie (0 = libre)
                while (suivant(&rail)){                                                      // idem pour les suivants
                       rail->Loop();
                       if (rail->demandeFermeture()){ddeFermeture = true;}
                        _etat = _etat + rail->etat();
                        }
                }
        if (barre) {testBarriere();}                                // si barrieres, on contr�le
        if (flash) {testFeux();}                                  // si feux routiers, on contr�le
    }
    else {testTriage();}                                           // on est en mode triage, suspension de l'etat du systeme
}

/********************************************************************************
TESTBARRIER : controle le fonctionnement de la barriere
algorithme :
    si (passage libre ET barriere inactive ET barriere fermee) on initialise l'ouverture et demande l'allumage des feux
    si (dde de fermeture ET barriere inactive ET barriere ouverte) on initialise la fermeture et demande l'allumage des feux
    si (barriere active) on poursuis la manoeuvre, les feux restent allumes
    si (barriere ouverte) on demande d'eteindre les feux
***********************************************************************************/

void passage::testBarriere() {
        if ((_etat == 0) && (!_barriere->actif()) && (_barriere->fermee())) {_barriere->ouverture(); spot = true; return;}        // si le passage est libre (tous les etats de voie a 0, on ouvre
        if ((!_barriere->actif()) && (ddeFermeture) && (_barriere->ouverte())) {_barriere->fermeture(); spot = true;return;}
        if (_barriere->actif()){_barriere->manoeuvre(); spot = true; return;}
        if (_barriere->ouverte()){spot = false;}                       // on attend la fin de la manoeuvre d'ouverture et on eteint les feux
}

/***********************************************************************
TESTLIGHT : gere les feux routiers
algorithme :
    si (barriere existe ET dde allumer feux)
        si (feux existent) allumer feux
        si (led tco existe) allumer led tco
    si (barriere existe ET dde eteindre feux)
        si (feux existent) eteindre feux
        si (led tco existe) eteindre led tco
    si (feux existent ET dde fermeture)
        allumer feux
        si (led tco existe) allumer led tco
    si (feux existent)
        eteindre feux
        si (led tco) eteindre led tco
********************************************************************************/

void passage::testFeux() {
  emis = false;
    if ((barre) && (spot)) {
            if (flash) {_feux->ON();}
            if (!ledCan) {actionS = "FERMEES"; emis = true; can->ecrireCode(LED_ON);ledCan = true;}
             return;}
    if (barre && !spot){
            if (flash) {_feux->OFF();}
            if (ledCan) {actionS = "OUVERTES";emis = true;can->ecrireCode(LED_OFF); ledCan = false;}
            return;}
    if (flash && (ddeFermeture)) {
            _feux->ON();
            if (!ledCan) {actionS = "FERMEES";emis = true;can->ecrireCode(LED_ON); ledCan = true;}
            return;}
    if (flash) {
            _feux->OFF();
            if (ledCan) {actionS = "OUVERTES";emis = true;can->ecrireCode(LED_OFF); ledCan = false;}
            }
}

void passage::emission() {if (emis) {Serial.print(F("PAN ==> TCO : BARRIERES "));Serial.println(actionS);}}

/***************************************************************************
TESTTRIAGE : gele le programme et ferme la barriere, le programme reprend la ou il en etait a la fin
algorithme :
    si (barriere existe ET barriere inactive ET barriere ouverte) intialisation de la fermeture, allumer les feux
    si (barriere existe ET barriere active) rester en manoeuvre, allumer les feux
    dde fermeture est vrai
    gerer les feux
**************************************************************************/

void passage::testTriage() {
    ddeFermeture = true;
    spot = true;
    testFeux();
    if (barre && (!_barriere->actif()) && (_barriere->ouverte())) {_feux->ON(); _barriere->fermeture();return;}
    if (barre && (_barriere->actif())){_barriere->manoeuvre();return;}
}




