#include "voie.h"

/*****************************************************
les constructeurs
***************************************************/

voie::voie(){}
voie::voie(char* nom):_nom(nom) {}

/*********************************************
le destructeur
************************************************/

voie::~voie(){}

/*********************************************************************************
CONFIG : configure la voie avec ces 4 zones
parametres :
    gaucheOuvre : adresse de la zone des capteurs gauches ouvrants
    gaucheFerme : adresse de la zone des capteurs gauches fermants
    droiteOuvre : adresse de la zone des capteurs droits ouvrants
    droitFerme  : adresse de la zone des capteurs droits fermants
**********************************************************************/

void voie::config (zone &gaucheOuvre, zone &gaucheFerme, zone &droiteOuvre, zone &droiteFerme){
canton[GAUCHE].zoneGauche.ouvre = &gaucheOuvre;
canton[GAUCHE].zoneGauche.ferme = &gaucheFerme;
canton[DROITE].zoneDroite.ouvre = &droiteOuvre;
canton[DROITE].zoneDroite.ferme = &droiteFerme;
}

/********************************************************************
CONFIG : idem, mais avec un nom
**************************************************************************/

void voie::config(zone &gaucheOuvre, zone &gaucheFerme, zone &droiteOuvre, zone &droiteFerme, char* nom){
_nom = nom;
canton[GAUCHE].zoneGauche.ouvre = &gaucheOuvre;
canton[GAUCHE].zoneGauche.ferme = &gaucheFerme;
canton[DROITE].zoneDroite.ouvre = &droiteOuvre;
canton[DROITE].zoneDroite.ferme = &droiteFerme;
}

/*********************************************************************************
CONFIG : configure la voie unidirectionnelle avec ces 3 zones
parametres :
    gaucheOuvre : adresse de la zone des capteurs gauches ouvrants
    gaucheFerme : adresse de la zone des capteurs fermants
    droiteOuvre : adresse de la zone des capteurs droits ouvrants
**********************************************************************/

void voie::config (zone &gaucheOuvre, zone &Ferme, zone &droiteOuvre){
canton[GAUCHE].zoneGauche.ouvre = &gaucheOuvre;
canton[GAUCHE].zoneGauche.ferme = &Ferme;
canton[DROITE].zoneDroite.ouvre = &droiteOuvre;
canton[DROITE].zoneDroite.ferme = &Ferme;
}

/********************************************************************
CONFIG : idem, mais avec un nom
**************************************************************************/

void voie::config(zone &gaucheOuvre, zone &Ferme, zone &droiteOuvre,char* nom){
_nom = nom;
canton[GAUCHE].zoneGauche.ouvre = &gaucheOuvre;
canton[GAUCHE].zoneGauche.ferme = &Ferme;
canton[DROITE].zoneDroite.ouvre = &droiteOuvre;
canton[DROITE].zoneDroite.ferme = &Ferme;
}

/******************************************************************************
PRINT : affiche la configuration de la voie (si mode console)
****************************************************************************/

void voie::print() {
    Serial.print(F("voie : "));Serial.println(nom());
    Serial.print(F("Direction en cours : ")); if (directionTrain == GAUCHE) {Serial.println(F(" vers la gauche"));} else {Serial.println(F(" vers la droite"));}
    Serial.print(F("Etat de la voie : ")); if(_etat > 0) {Serial.println(F("occupee"));} else {Serial.println(F("libre"));}
    canton[GAUCHE].zoneGauche.ouvre->print();
    canton[GAUCHE].zoneGauche.ferme->print();
    canton[DROITE].zoneDroite.ouvre->print();
    canton[DROITE].zoneDroite.ferme->print();
}

/***************************************************************************
ACTUALISER : mise a jour de tous les capteurs de la voie
********************************************************************************/

bool voie::actualiser() {
    canton[GAUCHE].zoneGauche.ouvre->actualiser();
    canton[GAUCHE].zoneGauche.ferme->actualiser();
    canton[DROITE].zoneDroite.ouvre->actualiser();
    canton[DROITE].zoneDroite.ferme->actualiser();
}

/***************************************************************************
LOOP : boucle sur les changements des capteurs de la voie et active le bon algorithme
******************************************************************************/

void voie::Loop() {
    actualiser();
    if (canton[GAUCHE].zoneGauche.ouvre->actif()) algoOuvre(GAUCHE);         // capteurs gauches ouvrants, zone gauche
    if (canton[GAUCHE].zoneGauche.ferme->actif()) algoFerme(GAUCHE);       // capteurs gauches fermants, zone gauche
    if (canton[DROITE].zoneDroite.ouvre->actif()) algoOuvre(DROITE);      // capteurs droits ouvrants,  zone droite
    if (canton[DROITE].zoneDroite.ferme->actif()) algoFerme(DROITE);    // capteurs droits fermants,  zone droite
}

/***********************************************************************************
ALGOOUVRE : algorithme des capteurs ouvrants
parametres :
    etat : adresse de la variable de demandes de passage
    sens : adresse du sens du train
    origine : le cote de la zone qui ex�cute l'algo (LEFT, RIGHT)
Algorithme :
    si la voie est libre (etat=0)=> {une demande de plus(etat++)}
        et {qu'on est sur la zone gauche => (le train vient donc de droite(sens=RIGHT)}
        sinon on est sur la zone droite et le train vient donc de gauche  (sens=LEFT)
    sinon si la zone et la direction du train sont les memes c'est une sortie on d�cremente si necessaire le nombre de demande (etat--)
          sinon c'est une nouvelle demande (etat++)
            si {on est sur la zone gauche le train vient donc de droite (sens=RIGHT)}
            sinon il vient de gauche (sens=LEFT)
***************************************************************************************/

void voie::algoOuvre(bool origine) {
    if (_etat == 0){_etat++; if (origine == GAUCHE) {directionTrain = DROITE;} else {directionTrain = GAUCHE;} return;}
    if (origine == directionTrain){
            if (_etat >0) {_etat--;}
            if (_etat == 0) {_demandeFermeture = false;}
            return;
            }
    else {_etat++; if (origine == GAUCHE) {directionTrain = DROITE;} else {directionTrain = GAUCHE;} return;}
}


/*********************************************************************************
ALGOFERME : algorithme des capteurs de fermeture
parametres :
    etat : adresse de la variable de demandes de passage
    sens : adresse du sens du train
    origine : le cote de la zone qui ex�cute l'algo (LEFT, RIGHT)
Algorithme :
    si il y a des demandes (etat>0) et si la zone est oppos�e au sens du train (sens) c'est une demande de fermeture
***************************************************************************************/

void voie::algoFerme(bool origine){
    if (_etat >0) {if (directionTrain != origine) {_demandeFermeture = true;return;}}
}
